package com.seeyon.apps.govdoc.debugger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.springframework.web.servlet.ModelAndView;

import com.seeyon.apps.govdoc.manager.GovdocLockManager;
import com.seeyon.apps.govdoc.manager.GovdocSummaryManager;
import com.seeyon.apps.govdoc.util.GovdocUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.controller.BaseController;
import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.organization.OrgConstants.Role_NAME;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.organization.principal.PrincipalManager;
import com.seeyon.ctp.util.FlipInfo;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.XMLCoder;
import com.seeyon.ctp.util.annotation.CheckRoleAccess;
import com.seeyon.ctp.organization.bo.V3xOrgAccount;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.bo.V3xOrgPrincipal;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.organization.principal.PrincipalManager;
import com.seeyon.v3x.edoc.domain.EdocSummary;

@CheckRoleAccess(roleTypes = {Role_NAME.AccountAdministrator,Role_NAME.EdocManagement})
public class GovdocDebuggerController extends BaseController {

	private static final Log LOGGER = CtpLogFactory.getLog(GovdocDebuggerController.class);
	
	public static final int pageSize = 999;//每页条数
	public static final int halfPageSize = 499;//每半页条数
	private OrgManager orgManager;
	private PrincipalManager principalManager;

	public void setOrgManager(OrgManager orgManager) {
	  this.orgManager = orgManager;
	}

	public void setPrincipalManager(PrincipalManager principalManager) {
	  this.principalManager = principalManager;
	}
	@Override
	public ModelAndView index(HttpServletRequest request,HttpServletResponse response) throws Exception {		
		ModelAndView mav = new ModelAndView("govdoc/debugger/index");
		User currentUser = AppContext.getCurrentUser();
		LOGGER.info(currentUser.getName() + " 进入公文调试界面。。。");
		return mav;
	}
	/**
     * 重置中文登录名用户密码为123456
     * @param request
     * @param response
     * @return
     * @throws Exception
     */
    public ModelAndView resetPassword4ChineseUserName(HttpServletRequest request, HttpServletResponse response) throws Exception {
        List<V3xOrgAccount> allAccounts = orgManager.getAllAccounts();
        Map<Long, V3xOrgPrincipal> v3xOrgPrincipalMap = new HashMap<Long, V3xOrgPrincipal>();
        for (V3xOrgAccount account : allAccounts) {
            Long accountId = account.getId();
            List<V3xOrgMember> allMembers = orgManager.getAllMembers(accountId);
            for (V3xOrgMember orgMember : allMembers) {
                V3xOrgPrincipal v3xOrgPrincipal = orgMember.getV3xOrgPrincipal();
                String loginName = v3xOrgPrincipal.getLoginName();
                if (isContainChinese(loginName) && orgMember.isValid()) {
                    v3xOrgPrincipal.setPassword("123456");
                    v3xOrgPrincipalMap.put(orgMember.getId(), v3xOrgPrincipal);
                }
            }
        }
        if (!v3xOrgPrincipalMap.isEmpty()) {
            List<V3xOrgPrincipal> v3xOrgPrincipalList = new ArrayList<V3xOrgPrincipal>();
            v3xOrgPrincipalList.addAll(v3xOrgPrincipalMap.values());
            principalManager.updateBatch(v3xOrgPrincipalList);
        }
        return null;
    }
    /**
     * 判断字符串中是否包含中文
     * @param str
     * 待校验字符串
     * @return 是否为中文
     * @warn 不能校验是否为中文标点符号
     */
    public static boolean isContainChinese(String str) {
        Pattern p = Pattern.compile("[\u4e00-\u9fa5]");
        Matcher m = p.matcher(str);
        if (m.find()) {
            return true;
        }
        return false;
    }

	/**
	 * 增加手工解锁入口
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	public ModelAndView unlock(HttpServletRequest request,HttpServletResponse response) throws Exception {
		AffairManager affairManager = (AffairManager)AppContext.getBean("affairManager");
		GovdocSummaryManager govdocSummaryManager = (GovdocSummaryManager)AppContext.getBean("govdocSummaryManager");
		GovdocLockManager govdocLockManager = (GovdocLockManager)AppContext.getBean("govdocLockManager");
		ModelAndView mav = new ModelAndView("govdoc/debugger/index");
		User currentUser = AppContext.getCurrentUser();
		CtpAffair affair = null;
		LOGGER.info(currentUser.getName() + " 进入公文解锁操作。。。");
		String param = request.getParameter("affairId");
		if(Strings.isNotBlank(param)) {
			Long affairId = Long.parseLong(param);
			try {
				affair = affairManager.get(affairId);
				govdocLockManager.unlock(affairId);
				LOGGER.info("解锁：affairId=" + affairId);
			} catch(Exception e) {
				LOGGER.error("解锁失败：affairId=" + affairId, e);
			}
		}
		
		param = request.getParameter("summaryId");
		if(Strings.isNotBlank(param)) {
			Long summaryId = Long.parseLong(param);
			try {
				EdocSummary summary = govdocSummaryManager.getSummaryById(summaryId, false);
				if(affair != null) {
			    	govdocLockManager.unlockAll(summary, affair.getMemberId());
				} else {
					govdocLockManager.unlockAll(summary);
				}
				LOGGER.info("解锁：summaryId=" + summaryId);
			} catch(Exception e) {
				LOGGER.error("解锁失败：summaryId=" +summaryId, e);
			}
		}
		
		return mav;
	}
	
	public ModelAndView upgrade(HttpServletRequest request,HttpServletResponse response) throws Exception {		
		User currentUser = AppContext.getCurrentUser();
		LOGGER.info(currentUser.getName() + " 进入公文升级操作。。。");
		response.setContentType("text/html;charset=UTF-8");
		try {
			upgradeAffairExtProp();
			
			upgradeWfCase();
			
			upgradeWfProcess();
			
			GovdocUtil.webAlertAndClose(response, "升级成功！");
		} catch(Exception e) {
			GovdocUtil.webAlertAndClose(response, "升级出错：" + e.getMessage());
			LOGGER.error("升级出错：" + e);
		}
		
		LOGGER.info(currentUser.getName() + " 公文升级操作成功！");
		
		return null;
	}

	@SuppressWarnings({ "unchecked", "unused" })
	private void upgradeAffairExtProp() throws Exception {
		LOGGER.info("开始升级公文Affair扩展字段。。。");
		Long start = System.currentTimeMillis();
		StringBuffer tempExt = null;
		//edoc_summary主表的数据参数
		Object doc_mark = null;
		Object send_unit = null;
		Object send_unit_id = null;
		
		//ctp_affair的数据
		Object pro_custom_dealwith = null;
		Object pro_custom_dealwith_memberid = null;
		Object pro_custom_dealwith_permission = null;
		Object pro_custom_dealwith_activitys = null;
		try {
			//填充数据有：文号、发文单位名称和ID、续办
			String sql = "select id,doc_mark,send_unit,send_unit_id,deadline_dateTime from edoc_summary where govdoc_type in (1,2,3,4)";
			String affairSql = "select id,pro_custom_dealwith,pro_custom_dealwith_memberid,pro_custom_dealwith_permission,pro_custom_dealwith_activitys,EXT_PROPS from ctp_affair where OBJECT_ID=?";
			String affair2Sql = "select id,EXT_PROPS from ctp_affair where OBJECT_ID=?";
			String updateAffairSql = "update ctp_affair set ext_props=? where id=?";
			boolean hasXuban = true;
			
			List<Map<String, Object>> summarys = null;
            List<Map<String, Object>> affairList = null;
			List<Object> params = null;
			
			List<Object[]> updateAffairList = new ArrayList<Object[]>(); 
			Object[] updateAffairObj = null;
			
			int page = 1;// 当前页数
            int total = -1;// 总共条数
            boolean canQuery = true;
            while (canQuery) {
            	//1.发文和发文单位有所有事项
    			//1.1查询edoc_summary，并拼接字符串
                FlipInfo fip = GovdocDebuggerSqlUtil.exeQueryByPage(sql, page, pageSize, total);
                if(total == -1) {
                    total = fip.getTotal() + 1;
                }
                summarys = fip.getData();
                if (summarys.size() < pageSize) {
                    canQuery = false;
                } else {
                    page++;
                }
                
                LOGGER.info("公文Affair扩展分页升级：当前第" + page + "页，每页" + pageSize + "条，共" + (total - 1) + "条。");
                
                for (int i = 0; i < summarys.size(); i++) {
    				tempExt = new StringBuffer("");
    				Long id = ((Number)summarys.get(i).get("id")).longValue();
    				doc_mark = summarys.get(i).get("doc_mark");
    				send_unit = summarys.get(i).get("send_unit");
    				send_unit_id = summarys.get(i).get("send_unit_id");
    				
    				//1.2 对应的ctp_affair 获取扩展字段
    				params = new ArrayList<Object>();
    				params.add(id);
    				if(hasXuban) {
	    				try {
	    					affairList = GovdocDebuggerSqlUtil.exeQuerySource(affairSql,params);
	    				} catch(Exception e) {
	    					LOGGER.error("ctp_affair中没有续办字段："+e.getMessage());
	    					try {
		    					affairList = GovdocDebuggerSqlUtil.exeQuerySource(affair2Sql,params);
		    					hasXuban = false;
	    					} catch(Exception e2) {
	    						LOGGER.error("", e);	
	    					}
	    				}
    				} else {
    					affairList = GovdocDebuggerSqlUtil.exeQuerySource(affair2Sql,params);
    				}
    				//1.3 修改扩展字段
    				for (int j = 0; j < affairList.size(); j++) {
    					 String affairExt = (String)affairList.get(j).get("ext_props");
    					 pro_custom_dealwith = affairList.get(j).get("pro_custom_dealwith");
    					 pro_custom_dealwith_memberid = affairList.get(j).get("pro_custom_dealwith_memberid");
    					 pro_custom_dealwith_permission = affairList.get(j).get("pro_custom_dealwith_permission");
    					 pro_custom_dealwith_activitys = affairList.get(j).get("pro_custom_dealwith_activitys");
    					 
    					 Map<String, Object> affairExtMap = (Map<String, Object>)XMLCoder.decoder(affairExt);
    	                if(affairExtMap == null){
    	                    affairExtMap = new HashMap<String, Object>();
    	                }
    					
    	                if(doc_mark != null && Strings.isNotBlank((String)doc_mark)){
    	                	affairExtMap.put("edoc_edocMark", doc_mark);
    					}
    					if(send_unit != null && Strings.isNotBlank((String)send_unit)){
    						affairExtMap.put("edoc_sendUnit", send_unit);
    					}
    					if(send_unit_id != null && Strings.isNotBlank((String)send_unit_id)){
    						affairExtMap.put("edoc_sendAccountId", send_unit_id);
    					}
    					//2.续办
    					if(pro_custom_dealwith != null) {
    						Integer value = ((Number)(pro_custom_dealwith)).intValue();
    						affairExtMap.put("pro_custom_dealwith", value==0 ? false : true);
    					}
    					if(pro_custom_dealwith_memberid != null) {
    						affairExtMap.put("pro_custom_dealwith_memberid", pro_custom_dealwith_memberid);
    					}
    					if(pro_custom_dealwith_permission != null) {
    						affairExtMap.put("pro_custom_dealwith_permission", pro_custom_dealwith_permission);
    					}
    					if(pro_custom_dealwith_activitys != null) {
    						affairExtMap.put("pro_custom_dealwith_activitys", pro_custom_dealwith_activitys);
    					}
    					
    					affairExt = XMLCoder.encoderMap(affairExtMap);
    					
    					updateAffairObj = new Object[2];
    					updateAffairObj[0] = affairExt;
    					updateAffairObj[1] = ((Number)affairList.get(j).get("id")).longValue();
    					updateAffairList.add(updateAffairObj);
    					
    					if(updateAffairList.size() == pageSize) {
    						GovdocDebuggerSqlUtil.exeUpdateBatch(updateAffairSql, updateAffairList);
    						updateAffairList.clear();
    					}
    				}
    			}
            }//while end
			
            if(Strings.isNotEmpty(updateAffairList)) {
            	GovdocDebuggerSqlUtil.exeUpdateBatch(updateAffairSql, updateAffairList);
				updateAffairList.clear();
            }
            
			LOGGER.info("公文Affair扩展字段升级完毕！执行耗时:"+(System.currentTimeMillis()-start)+"毫秒");
		} catch(Exception e) {
			LOGGER.error("升级公文Affair扩展字段错误：", e);
			throw new Exception(e);
		}
	}
	@SuppressWarnings("unchecked")
	private void upgradeWfCase() {
		LOGGER.info("开始升级公文流程AppName。。。");
		try {
			String wfSql = "select s.id as summary_id, wf.id as case_id,wf.caseobject as caseobject from edoc_summary s, wf_case_run wf where s.case_id = wf.id and s.govdoc_type in (1,2,3,4)";
			String updateSql = "update wf_case_run set caseobject = ? where id = ?";
	        Object[] updateObj = null;
	        List<Object[]> updateObjList = new ArrayList<Object[]>();
	        
			//目前签收流程结束后有被回退的流程，其它流程暂不处理
			String wfSqlHistory = "select s.id as summary_id, wf.id as case_id,wf.caseobject as caseobject from edoc_summary s, wf_case_history wf where s.case_id = wf.id and s.govdoc_type = 4";
			String updateSqlHistory = "update wf_case_history set caseobject = ? where id = ?";
	        Object[] updateObjHistory = null;
	        List<Object[]> updateObjHistoryList = new ArrayList<Object[]>();
	        
			boolean continueQuery = true;
			int page = 1;// 当前页数
	        int total = -1;// 总共条数
	        //分页查询主表记录
			while(continueQuery) {
				FlipInfo fip = GovdocDebuggerSqlUtil.exeQueryByPage(wfSql, page, halfPageSize, total);
				if(total == -1) {
	                total = fip.getTotal() + 1;
	            }
				LOGGER.info("公文流转中流程分页查询数。 当前第" + page + "页，每页" + halfPageSize + "条，共" + (total - 1) + "条。");
				List<Map<String, Object>> list = fip.getData();
	            if (list.size() < halfPageSize) {
	            	continueQuery = false;
	            	if(list.size() <= 0){
	            		break;
	            	}
	            } else {
	                page++;
	            }
	            for (Map<String, Object> onedata : list) {
	            	Object caseobject = onedata.get("caseobject");
	            	if(caseobject!=null && !"".equals(caseobject.toString())) {
	            		Long caseId = ((Number)onedata.get("case_id")).longValue();
			            byte[] newcaseobject = GovdocDebuggerSqlUtil.getNewWfCaseByAppName(caseobject, caseId);
			            if(newcaseobject != null) {
				            updateObj = new Object[2];
				            updateObj[0] = newcaseobject;
				            updateObj[1] = caseId;
				            updateObjList.add(updateObj);
				            
				            if(updateObjList.size() == halfPageSize) {
				            	GovdocDebuggerSqlUtil.exeUpdateBatch(updateSql, updateObjList);
								updateObjList.clear();
							}     
			            }
	            	}
	            }
			}
			if(Strings.isNotEmpty(updateObjList)) {
				GovdocDebuggerSqlUtil.exeUpdateBatch(updateSql, updateObjList);
				updateObjList.clear();
			}
			
			continueQuery = true;
			page = 1;// 当前页数
	        total = -1;// 总共条数
	        //分页查询主表记录
			while(continueQuery) {
				FlipInfo fip = GovdocDebuggerSqlUtil.exeQueryByPage(wfSqlHistory, page, halfPageSize, total);
				if(total == -1) {
	                total = fip.getTotal() + 1;
	            }
				LOGGER.info("公文已结束流程分页查询数。 当前第" + page + "页，每页" + halfPageSize + "条，共" + (total - 1) + "条。");
				List<Map<String, Object>> list = fip.getData();
	            if (list.size() < halfPageSize) {
	            	continueQuery = false;
	            	if(list.size() <= 0){
	            		break;
	            	}
	            } else {
	                page++;
	            }
	            for (Map<String, Object> onedata : list) {
	            	Object caseobject = onedata.get("caseobject");
	            	if(caseobject!=null && !"".equals(caseobject.toString())) {
	            		Long caseId = ((Number)onedata.get("case_id")).longValue();
			            byte[] newcaseobject = GovdocDebuggerSqlUtil.getNewWfCaseByAppName(caseobject, caseId);
			            if(newcaseobject != null) {
				            updateObjHistory = new Object[2];
				            updateObjHistory[0] = newcaseobject;
				            updateObjHistory[1] = caseId;
				            updateObjHistoryList.add(updateObjHistory);
				            
				            if(updateObjHistoryList.size() == halfPageSize) {
				            	GovdocDebuggerSqlUtil.exeUpdateBatch(updateSqlHistory, updateObjHistoryList);
								updateObjHistoryList.clear();
							}     
			            }
	            	}
	            }
			}
			if(Strings.isNotEmpty(updateObjHistoryList)) {
				GovdocDebuggerSqlUtil.exeUpdateBatch(updateSqlHistory, updateObjHistoryList);
				updateObjHistoryList.clear();
			}
			LOGGER.info("公文流程AppName升级完毕！");
		} catch(Exception e) {
			LOGGER.error("升级公文流程AppName错误：", e);
		}
	}

	@SuppressWarnings("unchecked")
	private void upgradeWfProcess() {
		LOGGER.info("开始升级公文流程收发员角色。。。");
		String wfSql = "select s.id as summary_id, wf.id as process_id,wf.processobject as processobject,s.edoc_type as edoc_type from edoc_summary s, wf_process_running wf where s.process_id = wf.id and s.state in (0,3)";
		String updateSql = "update wf_process_running set processobject = ? where id = ?";
        Object[] updateObj = null;
        List<Object[]> updateObjList = new ArrayList<Object[]>();

        try {
	        boolean continueQuery = true;
			int page = 1;// 当前页数
	        int total = -1;// 总共条数
	        //分页查询主表记录
			while(continueQuery) {
				FlipInfo fip = GovdocDebuggerSqlUtil.exeQueryByPage(wfSql, page, halfPageSize, total);
				if(total == -1) {
	                total = fip.getTotal() + 1;
	            }
				LOGGER.info("公文公文流程收发员角色分页查询数。 当前第" + page + "页，每页" + halfPageSize + "条，共" + (total - 1) + "条。");
				List<Map<String, Object>> list = fip.getData();
	            if (list.size() < halfPageSize) {
	            	continueQuery = false;
	            	if(list.size() <= 0){
	            		break;
	            	}
	            } else {
	                page++;
	            }
	            for (Map<String, Object> onedata : list) {
	            	if(onedata.get("edoc_type") == null) {
	            		continue;
	            	}
	            	
	            	Integer edocType = ((Number)onedata.get("edoc_type")).intValue();
	            	Object processobject = onedata.get("processobject");
	            	if(processobject!=null && !"".equals(processobject.toString())) {
	            		Long processId = Long.parseLong(onedata.get("process_id").toString());
			            String newprocessobject = GovdocDebuggerSqlUtil.getNewWfProcessXmlByRole(processobject, edocType, processId);
			            if(Strings.isNotBlank(newprocessobject)) {
				            updateObj = new Object[2];
				            updateObj[0] = GovdocDebuggerSqlUtil.transBpmObjectToByte(newprocessobject);
				            updateObj[1] = processId;
				            updateObjList.add(updateObj);
				            
				            if(updateObjList.size() == halfPageSize) {
				            	GovdocDebuggerSqlUtil.exeUpdateBatch(updateSql, updateObjList);
								updateObjList.clear();
							}     
			            }
	            	}
	            }
			}
			if(Strings.isNotEmpty(updateObjList)) {
				GovdocDebuggerSqlUtil.exeUpdateBatch(updateSql, updateObjList);
				updateObjList.clear();
			}
			LOGGER.info("公文流程收发员角色升级完毕！");
		} catch(Exception e) {
			LOGGER.error("升级公文流程收发员角色错误：", e);
		}		
	}
}
