package com.seeyon.apps.govdoc.debugger;

import java.lang.management.ManagementFactory;
import java.lang.management.RuntimeMXBean;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.Strings;

public class GovdocDebuggerHelper {

	private static OrgManager orgManager = (OrgManager)AppContext.getBean("orgManager");
	
	private static final Log LOGGER = LogFactory.getLog(GovdocDebuggerHelper.class);
	
	public static Map<Integer, List<GovdocDebuggerBO>> debugerMap = new HashMap<Integer, List<GovdocDebuggerBO>>();
	
	public static void startDebugger() {
		List<GovdocDebuggerBO> list = new ArrayList<GovdocDebuggerBO>();
		list.add(new GovdocDebuggerBO(false, System.currentTimeMillis()));
		debugerMap.put(getSysProcessID(), list);
	}
	
	public static void addDebugger(boolean enabled) {
		if(debugerMap.get(getSysProcessID()) == null) {
			debugerMap.put(getSysProcessID(), new ArrayList<GovdocDebuggerBO>()); 
		}
		debugerMap.get(getSysProcessID()).add(new GovdocDebuggerBO(enabled, System.currentTimeMillis()));
	}
	
	public static void addDebugger(boolean enabled, String msg) {
		debugerMap.get(getSysProcessID()).add(new GovdocDebuggerBO(enabled, msg, System.currentTimeMillis()));
	}
	
	public static void addDebugger(String msg) {
		debugerMap.get(getSysProcessID()).add(new GovdocDebuggerBO(true, msg, System.currentTimeMillis()));
	}
	
	public static void endDebugger(int code, String msg, String userName, String subject) {
		int sysProcessId = getSysProcessID();
		try {
			long lend = System.currentTimeMillis();
			List<GovdocDebuggerBO> list = debugerMap.get(getSysProcessID());
			if(Strings.isNotEmpty(list) && list.size() > 1) {
				GovdocDebuggerBO startBO = list.get(0);
				if(code == GovdocDebuggerConstant.ACTION_NEW) {//公文拟文
					LOGGER.info(userName + "新建公文方法" + msg + "总耗时：" + (lend - startBO.getSysTimes()));
				} else if(code == GovdocDebuggerConstant.ACTION_SEND) {//公文发送
					LOGGER.info(userName + "发送公文《"+subject+"》方法" + msg + "总耗时：" + (lend - startBO.getSysTimes()));
				} else if(code == GovdocDebuggerConstant.ACTION_VIEW) {//公文查看
					LOGGER.info(userName + "打开公文《"+subject+"》方法" + msg + "总耗时：" + (lend - startBO.getSysTimes()));
				} else if(code == GovdocDebuggerConstant.ACTION_FINISH) {//公文提交
					LOGGER.info(userName + "提交公文《"+subject+"》方法" + msg + "总耗时：" + (lend - startBO.getSysTimes()));
				}
				for(int i=1; i<list.size(); i++) {
					GovdocDebuggerBO lastBO = list.get(i-1);
					GovdocDebuggerBO debugBO = list.get(i);
					if(debugBO.isEnabled()) {
						LOGGER.info(debugBO.getMsg() + "耗时：" + (debugBO.getSysTimes() - lastBO.getSysTimes()));
					}
				}
			} 
		} catch(Exception e) {
			LOGGER.info("调试错误请忽略", e);
		} finally {
			debugerMap.remove(sysProcessId);
		}
	}

    public static final int getSysProcessID() {  
        RuntimeMXBean runtimeMXBean = ManagementFactory.getRuntimeMXBean();
        return Integer.valueOf(runtimeMXBean.getName().split("@")[0]).intValue();  
    }
	
}
