package com.seeyon.apps.govdoc.debugger;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;

import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.util.FlipInfo;
import com.seeyon.ctp.util.JDBCAgent;
import com.seeyon.ctp.util.Strings;

public class GovdocDebuggerSqlUtil {
	
	private static final Log LOGGER = CtpLogFactory.getLog(GovdocDebuggerController.class);
	
	public static boolean selectRoleFlag = false;
	private static List<Long> roleIdList = new ArrayList<Long>();
	
	/**
     * 分页查询
     * @param sql
     * @param page
     * @param pageSize
     * @param total 如果total > 0则不进行总数查询
     * @return
     * @throws Exception 
     * @throws InstallException
     * @throws SQLException
     */
    public static FlipInfo exeQueryByPage(String sql, int page, int pageSize, int total) throws Exception{
        
        JDBCAgent dba = new JDBCAgent();
        
        FlipInfo fi = null;
        try {
            fi = exeQueryByPage(dba, sql, page, pageSize, total);
        } catch (Exception e) {
            throw e;
        }finally {
            dba.close();
        }
        
        return fi;
    }
    
    /**
     * 使用已有的 连接执行sql 
     * 
     * @param dba
     * @param sql
     * @param page
     * @param pageSize
     * @param total
     * @return
     * @throws Exception
     *
     * @Since A8-V5 7.0SP1
     * @Author      : xuqw
     * @Date        : 2018年9月19日下午4:41:59
     *
     */
    public static FlipInfo exeQueryByPage(JDBCAgent dba, String sql, int page, int pageSize, int total) throws Exception{

        FlipInfo fpi = new FlipInfo();
        fpi.setPage(page);
        
        if(total < 0){
            fpi.setNeedTotal(true);
        }else {
            fpi.setTotal(total);
            fpi.setNeedTotal(false);
        }
        
        fpi.setSize(pageSize);
        try {
            dba.findByPaging(sql, fpi);
        } catch (Exception e) {
            LOGGER.error("执行SQL分页查询失败", e);
            throw e;
        }
        
        return fpi;
    }
    
    /**
     * 执行数据库查询
     * @param sql
     * @param params
     * @return
     * @throws Exception
     */
    @SuppressWarnings({ "unchecked", "rawtypes" })
    public static List<Map<String, Object>> exeQuerySource(String sql, List params) throws Exception {
        List<Map<String, Object>> result = new ArrayList<Map<String, Object>>();
        JDBCAgent jdbcAgent = new JDBCAgent();
        try {
            if(Strings.isNotEmpty(params)) {
                jdbcAgent.execute(sql, params);
            } else {
                jdbcAgent.execute(sql);
            }
            result = (List<Map<String, Object>>)jdbcAgent.resultSetToList();
        } catch (Exception e) {
        	LOGGER.error("执行SQL查询失败", e);
        	throw e;
        } finally {
            jdbcAgent.close();
        }
        return result;
    }
	
    public static List<Map<String, Object>> exeQuerySource(String sql) throws Exception {
    	return exeQuerySource(sql, null);
    }
    
    public static void exeUpdateBatch(String sql, List<Object[]> parameters) throws Exception {
        JDBCAgent jdbcAgent = new JDBCAgent();
        jdbcAgent.batch1Prepare(sql);
        List<Object> params = null;
        for (Object[] objects : parameters) {
            params = new ArrayList<Object>(objects.length);
            for (int i = 0; i < objects.length; i++) {
                params.add(objects[i]);
            }
            jdbcAgent.batch2Add(params);
        }
        jdbcAgent.batch3Execute();
        jdbcAgent.close();
    }
    
    @SuppressWarnings({ "rawtypes", "unchecked" })
    public static byte[] getNewWfCaseByAppName(Object caseobject, Object objectId) {
		Object obj = parseWfProcessXml(caseobject, objectId);
		if(obj != null) {
            Map map = (Map)obj;
            if(null!=map.get("datamap")) {
			    Map dataMap = (Map)map.get("datamap");
			    String appName = dataMap.get("appName").toString();
				if("collaboration".equals(appName)) {
					dataMap.put("appName", "edoc");
					ByteArrayOutputStream byt = null;
					ObjectOutputStream oos = null;
					try {
						byt = new ByteArrayOutputStream();
						oos = new ObjectOutputStream(byt);
						oos.writeObject(map);					
					} catch(IOException e) {
						LOGGER.error("流程更新appName时出错 objectId=" + objectId, e);
						return null;
					} finally {
						if(oos != null){
		                    try {
		                    	oos.close();
		                    } catch (IOException e) {
		                    	LOGGER.error("", e);
		                    }
		                }
						if(byt != null){
		                    try {
		                    	byt.close();
		                    } catch (IOException e) {
		                    	LOGGER.error("", e);
		                    }
		                }
					}
					byte[] bytes = byt.toByteArray();
					return bytes;
				}
			}
        }
		return null;
	}
        
    public static String getNewWfProcessXmlByRole(Object caseobject, Integer edocType, Object objectId) {
		Object obj = parseWfProcessXml(caseobject, objectId);
		if(obj != null) {
			String oldprocessXml = obj.toString();
			if(Strings.isNotBlank(oldprocessXml)) {
				String processXml = oldprocessXml;
				if (processXml.contains("Departmentexchange")) {//老公文部门公文收发员
					if (edocType == 0 || edocType == 2) {
						processXml = oldprocessXml.replaceAll("Departmentexchange", "DepartmentGovdocSend").replaceAll("公文收发员", "公文送文员");
					} else if (edocType == 1) {
						processXml = oldprocessXml.replaceAll("Departmentexchange", "DepartmentGovdocRec").replaceAll("公文收发员", "公文收文员");
					}
				}
				if (processXml.contains("Accountexchange")) {//老公文单位公文收发员
					if (edocType == 0 || edocType == 2) {
						processXml = oldprocessXml.replaceAll("Accountexchange", "AccountGovdocSend").replaceAll("公文收发员", "公文送文员");
					} else if (edocType == 1) {
						processXml = oldprocessXml.replaceAll("Accountexchange", "AccountGovdocRec").replaceAll("公文收发员", "公文收文员");
					}
				}
				if (processXml.contains("AccountAndDepartment")) {//新公文多单位多部门公文收发员匹配
					if(!selectRoleFlag) {//只查询一次
	            		setAllRoleId();
	            	}
					
					String endProcessXml = processXml;
					while(endProcessXml.indexOf("AccountAndDepartment") >= 0) {
						int index = endProcessXml.indexOf("AccountAndDepartment");
						int eIndex = index + endProcessXml.substring(index).indexOf("\"");
						if(index < 0 || eIndex < 0) {
							break;
						}
						String partyId = endProcessXml.substring(index, eIndex);
						String[] partyIdValue = partyId.split("[@|#]");
			    		if(partyIdValue != null) {
			    			if(partyIdValue.length == 4) {
			    				String fieldName = partyIdValue[2];
			    				String roleName = partyIdValue[3];
			    				try {
			    					Long roleId = Long.parseLong(roleName);
			    					if(!roleIdList.contains(roleId)) {
			    						String accountExchangeRole = "";
			    						if("发文单位".equals(fieldName)) {
			    							accountExchangeRole = "AccountGovdocSend";
			    						} else {
			    							accountExchangeRole = "AccountGovdocRec";
			    						}
			    						String newpartyId = partyId.replace(roleName, accountExchangeRole);
			    						processXml.replace(partyId, newpartyId);
			    						endProcessXml = endProcessXml.substring(eIndex, endProcessXml.length());
			    					} else {
			    						endProcessXml = endProcessXml.substring(eIndex, endProcessXml.length());
					    				continue;
			    					}
			    				} catch(NumberFormatException nfe) {
			    					endProcessXml = endProcessXml.substring(eIndex, endProcessXml.length());
			    					continue;
			    				}
			    			} else {
			    				endProcessXml = endProcessXml.substring(eIndex, endProcessXml.length());
			    				continue;
				    		}
			    		} else {
			    			endProcessXml = endProcessXml.substring(eIndex, endProcessXml.length());
		    				continue;
			    		}
					}
				}
				
				if(!processXml.equals(oldprocessXml)) {
					LOGGER.info("需要替换流程processXml objectId=" + objectId);
					LOGGER.info("oldprocessXml=" + oldprocessXml);
					LOGGER.info("processXml=" + processXml);
					return processXml;
				}
			}
        }
		return null;
	}	

	public static Object parseWfProcessXml(Object caseobject, Object objectId) {
		byte[] byteArray = null;
		String dbms = JDBCAgent.getDBType();
		Object obj = null;
		InputStream is = null;
        ObjectInputStream ois = null;
        try {
			if ("MySQL".equalsIgnoreCase(dbms)) {
				byteArray = (byte[])caseobject;
				if (null != byteArray) {
					is = new ByteArrayInputStream(byteArray);
				}
			} else if ("Microsoft SQL Server".equalsIgnoreCase(dbms) || "sqlserver".equalsIgnoreCase(dbms) || "Oracle".equalsIgnoreCase(dbms)) {
				java.sql.Blob blob = (java.sql.Blob)caseobject;
				is = blob.getBinaryStream();
			}
			if(is != null) {
				ois = new ObjectInputStream(is);
				obj = ois.readObject();
			}
		} catch (Throwable e) {
			LOGGER.error("读取流程大字段出现异常，可能未读到 objectId= " + objectId + " 异常信息：" + e.getMessage(), e);
        	obj = null;
        } finally {
            if(is != null){
                try {
                	is.close();
                } catch (IOException e) {
                	LOGGER.error("", e);
                }
            }
            if(ois != null){
                try {
                	ois.close();
                } catch (IOException e) {
                	LOGGER.error("", e);
                }
            }
        }
        return obj;
	}
	
	public static byte[] transBpmObjectToByte(String xml){
		ByteArrayOutputStream byt = null;
		ObjectOutputStream oos = null;
		try {
			byt = new ByteArrayOutputStream();
			oos = new ObjectOutputStream(byt);
			oos.writeObject(xml);					
		} catch(IOException e) {
			LOGGER.error("", e);
		} finally {
			if(oos != null){
                try {
                	oos.close();
                } catch (IOException e) {
                	LOGGER.error("", e);
                }
            }
			if(byt != null){
                try {
                	byt.close();
                } catch (IOException e) {
                	LOGGER.error("", e);
                }
            }
		}
		byte[] bytes = byt.toByteArray();
		return bytes;
	}
	
	public static void setAllRoleId() {
		roleIdList.clear();
		selectRoleFlag = true;
		try {
			String roleSql = "select id from org_role where code in ('Departmentexchange','DepartmentGovdocSend','DepartmentGovdocRec','Accountexchange','AccountGovdocSend','AccountGovdocRec')";
			List<Map<String, Object>> roles = exeQuerySource(roleSql,null);
			//循环每个公文模板
			for (Map<String, Object> map : roles) {
				roleIdList.add(((Number)map.get("id")).longValue());
			}
		} catch(Exception e) {
			LOGGER.error("获取所有公文交换员出错：", e);
		}
	}
	
}
